//
// controlled.letter_array3.44.c
//
// Changes from v2: added interrupt support, added line animation for idle mode, added some letters
// Changes from v1: removed unnecessary commands, moved serial commands into a separate header file.
// Gets a character from user and echoes it. If the character is a lowercase
// letter a-t, blinks the corresponding LED (1-20). Uses PB0 for ouput
// and PB1 for input
//
// Based on:
// hello.array.44.c
// Charlieplex LED array hello-world
//
// Neil Gershenfeld
// 11/13/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h> 

#include "led_display.h"
#include "led_serial.h"

#define serial_interrupt (1 << PCIE1)
#define serial_interrupt_pin (1 << PCINT8)
#define interrupt_vector PCINT1_vect // PCINT11-8 are on the PCINT1_vector
#define mask_general_interrupt_register GIMSK
#define mask_pin_change_register PCMSK1


//uint8_t led_matrix[num_leds];
//uint8_t volatile nCurrentLed;
uint8_t matrix_on;


void led_map(char led, uint8_t delay) {
	//
	// Call the respective function for each character
	//
	switch (led) {
		case 'a': show_letter_a(delay); break;
		case 'b': show_letter_b(delay); break;
		case 'c': show_letter_c(delay); break;
		case 'd': show_letter_d(delay); break;
		case 'e': show_letter_e(delay); break;
		case 'f': show_letter_f(delay); break;
		case 'g': show_letter_g(delay); break;
		case 'n': show_letter_n(delay); break;
		case '0': show_line_0(delay); break;
		case '1': show_line_1(delay); break;
		case '2': show_line_2(delay); break;
		case '3': show_line_3(delay); break;
		case '4': show_line_4(delay); break;
		case '@': show_all(delay); break;
		case '#': show_none(delay); break;
		case '(': matrix_on = 1; break;
		case ')': matrix_on = 0; break;
		default: show_checkerboard(delay); break;
	}

} // end led_on()


void matrix_idle_display(uint8_t delay) {
	//
	// Display this while not handling interrupts
	//
	led_map('0', delay);
	led_map('1', delay);
	led_map('2', delay);
	led_map('3', delay);
	led_map('4', delay);
	led_map('3', delay);
	led_map('2', delay);
	led_map('1', delay);
}


ISR(PCINT1_vect) {
	//
	// pin change interrupt handler
	//
	static char chr;
	get_char(&serial_pins, serial_pin_in, &chr);
	//
	// turn matrix on
	//
	matrix_on = 1;
	//
	// display
	//
	led_map(chr, 100);
	
	// Can't output until I fix the wire!
	//static char message[] PROGMEM = "hello.ftdi.44.echo.interrupt.c: you typed ";
	//put_flash_string(&serial_port, serial_pin_out, (PGM_P) message);
	//put_char(&serial_port, serial_pin_out, chr);
	//put_char(&serial_port, serial_pin_out, 10); // new line
}


int main(void) {
	//
	// main
	//
	//static char chr;
	//static char buffer[max_buffer] = {0};
	int delay = 100;
	//
	// set clock divider to /1
	//
	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	//
	// set up pin change interrupt on input pin
	//
	set(mask_general_interrupt_register, serial_interrupt);
	set(mask_pin_change_register, serial_interrupt_pin);
	sei();	// enable interrupts
	//
	// initialize output pins
	//
	set(serial_port, serial_pin_out);
	output(serial_direction, serial_pin_out);
	//
	// main loop
	//
	matrix_on = 1;
	//
	while (1) {
		// get_char(&serial_pins, serial_pin_in, &chr);
		// put_char(&serial_port, serial_pin_out, chr);
		
		if (matrix_on) {
			// if matrix is turned on, show line animation
			matrix_idle_display(delay);
		}
	}
}