#ifndef LED_DISPLAY_H
#define LED_DISPLAY_H

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_direction DDRA

#define A (1 << PA1) // row 1
#define B (1 << PA2) // row 2
#define C (1 << PA3) // row 3
#define D (1 << PA4) // row 4
#define E (1 << PA5) // row 5


void flash(uint8_t from, uint8_t to, uint8_t delay) {
	//
	// source from, sink to, flash
	//
	static uint8_t i;
	set(led_port,from);
	clear(led_port,to);
	output(led_direction,from);
	output(led_direction,to);
	for (i = 0; i < delay; ++i)
		led_delay();
	input(led_direction,from);
	input(led_direction,to);
}

//
// The following functions draw their respective letters
//

void show_letter_a(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		//flash(B,A,1);
		flash(C,A,1);
		flash(D,A,1);
		//flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		//flash(B,E,1);
		//flash(C,E,1);
		flash(D,E,1);
	}
}

void show_letter_b(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(B,A,1);
		flash(C,A,1);
		flash(D,A,1);
		//flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		//flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		//flash(D,E,1);
	}
}

void show_letter_c(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		
		//flash(B,A,1);
		flash(C,A,1);
		flash(D,A,1);
		flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		//flash(E,B,1);
	
		flash(A,C,1);
		//flash(B,C,1);
		//flash(D,C,1);
		//flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		//flash(E,D,1);
	
		//flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		flash(D,E,1);
	}
}

void show_letter_d(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		flash(B,A,1);
		flash(C,A,1);
		flash(D,A,1);
		//flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		//flash(B,C,1);
		//flash(D,C,1);
		flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		//flash(D,E,1);
	}
}

void show_letter_e(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		flash(B,A,1); 
		flash(C,A,1);
		flash(D,A,1);
		flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		//flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		//flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		//flash(E,D,1);
	
		flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		flash(D,E,1);
	}
}

void show_letter_f(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		flash(B,A,1); 
		flash(C,A,1);
		flash(D,A,1);
		flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		//flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		//flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		//flash(E,D,1);
	
		flash(A,E,1);
		//flash(B,E,1);
		//flash(C,E,1);
		//flash(D,E,1);
	}
}

void show_letter_g(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		//flash(B,A,1); 
		flash(C,A,1);
		flash(D,A,1);
		//flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		//flash(E,B,1);
	
		flash(A,C,1);
		//flash(B,C,1);
		flash(D,C,1);
		flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		//flash(C,D,1);
		flash(E,D,1);
	
		//flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		//flash(D,E,1);
	}
}

void show_letter_n(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		flash(B,A,1); 
		//flash(C,A,1);
		//flash(D,A,1);
		flash(E,A,1);
	
		flash(A,B,1);
		//flash(C,B,1);
		//flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		//flash(D,C,1);
		flash(E,C,1);
	
		flash(A,D,1);
		//flash(B,D,1);
		flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		//flash(B,E,1);
		//flash(C,E,1);
		flash(D,E,1);
	}
}

void show_checkerboard(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
	
		flash(B,A,1); 
		//flash(C,A,1);
		flash(D,A,1);
		//flash(E,A,1);
	
		//flash(A,B,1);
		flash(C,B,1);
		//flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		//flash(B,C,1);
		flash(D,C,1);
		//flash(E,C,1);
	
		//flash(A,D,1);
		flash(B,D,1);
		//flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		//flash(B,E,1);
		flash(C,E,1);
		//flash(D,E,1);
	}
}

void show_line_0(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(B,A,1); 
		flash(C,A,1);
		flash(D,A,1);
		flash(E,A,1);
	}
}

void show_line_1(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(A,B,1);
		flash(C,B,1);
		flash(D,B,1);
		flash(E,B,1);
	}
}

void show_line_2(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		flash(E,C,1);
	}
}

void show_line_3(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(A,D,1);
		flash(B,D,1);
		flash(C,D,1);
		flash(E,D,1);
	}
}

void show_line_4(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		flash(D,E,1);
	}
}

void show_all(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		flash(B,A,1); 
		flash(C,A,1);
		flash(D,A,1);
		flash(E,A,1);
	
		flash(A,B,1);
		flash(C,B,1);
		flash(D,B,1);
		flash(E,B,1);
	
		flash(A,C,1);
		flash(B,C,1);
		flash(D,C,1);
		flash(E,C,1);
	
		flash(A,D,1);
		flash(B,D,1);
		flash(C,D,1);
		flash(E,D,1);
	
		flash(A,E,1);
		flash(B,E,1);
		flash(C,E,1);
		flash(D,E,1);
	}
}

void show_none(uint8_t delay) {
	uint8_t i;
	
	for (i=0; i<delay; ++i) {
		// nothing!
	}
}

#endif